"""Helper functions

Consists of functions to typically be used within templates, but also
available to Controllers. This module is available to templates as 'h'.
"""

# import webhelpers html tags. These can be patched as we want. However, I may
# in the future use the formencode field functions, wrapping them up in simple
# functions here. That way we can add a bit more intelligence in our web forms.
from webhelpers.html.tags import form, end_form, text, password, submit
from webhelpers.html.tags import stylesheet_link, javascript_link, ModelTags, hidden, select
from webhelpers.html.tags import checkbox, image, textarea

# url allows us to generate URLs for specific controllers on disk, which
# makes it easier to reorganise routes without having to touch template code:
# Those of you who are used to url should use this in exactly the same way.
# you can also wrap normal URLs in url, and it will prepend any hosting prefix
# to the generated URL.
#
# Bottom line: ALL generated URLs MUST be wrapped in a call to url():
from pylons import url, config

# Roles information
from sloecode.model.membership import PROJECT_ROLES
from sloecode.model.site_role import SITE_ROLES

# a simple link_to implementation that handles a confirm kwarg that prints
# very simple javascript asking the user if they really want to continue.
from webhelpers.html.tags import link_to as _link_to
def link_to(label, url, **kwargs):
    """A patched version of the link_to function that accepts a confirm
    parameter.
    """
    # only add JS if the user has not already put something in onclick handler
    if 'confirm' in kwargs and 'onclick' not in kwargs:
        kwargs['onclick'] = 'return window.confirm("%s")' % (kwargs['confirm'])
        del kwargs['confirm']
    return _link_to(label, url, **kwargs)


# this allows controllers to show informative messages to users:
from webhelpers.pylonslib import Flash as _Flash
#
# To use this from a controller (assuming this module is imported as 'h'), do
# this:
#
# h.flash("Some Message Text")
#
# You may also include a category:
#
# h.flash("Some Message Text", "warning")
#
# Allowed categorioes are warning, notice, error, success. The default
# category is 'notice'.
flash = _Flash()

# import helper functions that let templates determine if the user has
# certain permissions:
from sloecode.lib.security import has_site_role, has_project_role
# import site role names as well:
from sloecode.model.site_role import USER_ADMIN, PROJECT_ADMIN


# Help support:
def help_link(label, target):
    """
    Returns HTML code that creates a link to a help topic. Help topics are
    specified by the 'target' parameter. The topic should be the path of a
    help file, relative to the help_pages directory, either with, or without
    the '.html' extension.
    """
    if target[-5:].lower != '.html':
        target += '.html'
    if target[0] == '/':
        target = target[1:]

    link = url('/help/' + target)
    return link_to(label, link, class_='help_link')

def allow_password_modification():
    # No password modification if PAM is used for authentication.
    use_pam = config.get('use_pam', False)
    return not use_pam

def allow_user_creation():
    # Don't allow user creation when using PAM for authentication.
    print 'allow_user_creation'
    use_pam = config.get('use_pam', False)
    return not use_pam
