#!/usr/bin/env python

"""
Script to upgrade a live database schema.
"""

from sys import argv
if len(argv) != 2:
    print "Usage: upgrade_database.py /path/to/sloecode/config.ini"
    exit(1)

from sqlalchemy import engine_from_config
from sloecode.lib.readconf import get_conf
import os
import re

def main():
    patch_dir = 'dbpatches'
    config = get_conf('app:main', argv[1])
    Engine = engine_from_config(config)
    Connection = Engine.connect()

    # we now have the connection and engine configured. First, get the current
    # database version number from the database:
    dbversion = 0
    try:
        result = Connection.execute(r"SELECT MAX(version) from database_version;").fetchone()
        dbversion = result[0]
    except:
        pass
    
    print "Database version is currently:", dbversion

    # Get the current new database version from trunk - this is our stop point:
    enddbversion = int(open('DBVERSION','r').read())
    print "Upgrading database to version:", enddbversion

    # anything to do?
    if enddbversion <= dbversion:
        print "Nothing to do!"
        exit(0)

    print "Database type is:", Engine.name
    backend = Engine.name

    patch_re = re.compile(r'^sloecode-(\d+)(-%s)?.sql' % (backend))

    # store all patches in dictionary according to patch number:
    patches = {}

    # find all patch files in dbpatches folder.
    # TODO: Ensure we're looking at the correct folder. How will this work with
    # debian package upgrades? We'll need to store these in the filesystem somewhere!
    for patch_file in os.listdir(patch_dir):
        match = patch_re.match(patch_file)
        if match:
            groups = match.groups()
            patch_number = int(groups[0])
            # not that we care, but we can tell if the patch is backend specific here.
            # backend-generic patches will have None in 'groups[1]'.
            patches[patch_number] = patch_file

    # make sure we have all the patch files we need. Bail if we're missing anything:
    for i in range(dbversion+1, enddbversion+1):
        if i not in patches:
            print "Error: Cannot find patch to upgrade schema from version %d to %d." % (i-1, i)
            exit(1)

    # loop through every patch number, and apply patch WITHIN A DATABASE TRANSACTION!
    trans = Connection.begin()
    try:
        for i in range(dbversion+1, enddbversion+1):
            apply_patch(Connection, os.path.join(patch_dir, patches[i]), i)
        trans.commit()
    except:
        trans.rollback()
        raise
    #
    # If the patch fails, roll back the transaction. Perhaps this should be a nested
    # transaction, so we can roll back the entire operation?
    #
    # Also, we need to know what kind of back-end we're dealing with. For any
    # particular version number, the patch will either be sloecode-NN.sql or
    # sloecode-NN-mysql|sqlite.sql

def apply_patch(conn, patch_file, version):
    trans = conn.begin()
    try:
        print "Applying patch:", patch_file
        conn.execute(open(patch_file).read())
        # update the DB version:
        conn.execute('INSERT INTO database_version (version) VALUES (%d)' % version)
        trans.commit()
    except:
        trans.rollback()
        raise

if __name__ == '__main__':
    main()
